--[[

BetterTradeSkill
	by Mattperry (http://somewhere.fscked.org)
	copyright: GNU General Public License (http://www.gnu.org/copyleft/gpl.html)

Modifies the TradeSkill frame to have "Queue" buttons, so you can queue up
multiple trade skill items and have them created in sequence automatically.
For example, if I have 14 silver ore and 29 iron ore, I can queue them all
up and walk away while it smelts the silver then iron.

Version history:
v1: 2005mar02
	initial release

--]]

local P = {};
BetterTradeSkill = P;

-- SkillQueue is both a lookup table and a linked list.  This allows me to easily find an entry by item
-- while still preserving the order they were created in.
local SkillQueue = {};
local SkillQueueHead = nil;
local SkillQueueTail = nil;
local CastingSkill = false;

-- Internal functions

function P.OnLoad()
	this:RegisterEvent("CHAT_MSG_SPELL_TRADESKILLS");
	this:RegisterEvent("TRADE_SKILL_SHOW");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_START");
end

function P.OnEvent()
	if (event == "CHAT_MSG_SPELL_TRADESKILLS") then
		for item in string.gfind(arg1, "You create (.+).") do
			P.HandleCreated(item);
			return;
		end
	elseif (event == "TRADE_SKILL_SHOW") then
		-- They opened/switched the tradeskill window.  Make sure we clear out the skill queue in case it
		-- was left unempty before.  (This also handles the case of switching to another TradeSkill window)
		SkillQueueHead = nil;
		SkillQueueTail = nil;
		SkillQueue = {};
		CastingSkill = false;
	elseif (event == "SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED") then
		CastingSkill = false;
	elseif (event == "SPELLCAST_START") then
		CastingSkill = true;
	end
end

function P.QueueEverything()
	local numSkills = GetNumTradeSkills();
	for index = 1,numSkills do
		local item, amount = P.GetTradeSkillInfo(index);
		if (amount > 0) then
			P.AddToQueue(item, index, amount);
		end
	end
end

function P.Queue(amount)
	local index = GetTradeSkillSelectionIndex();
	local item, maxAmount = P.GetTradeSkillInfo(index);

	if (amount > maxAmount) then amount = maxAmount; end

	if (amount > 0) then
		P.AddToQueue(item, index, amount);
	end
end

function P.AddToQueue(item, index, amount)
	if not SkillQueue[item] then
		SkillQueue[item] = {item = item};

		-- add it to the skill queue
		if not SkillQueueHead then
			SkillQueueHead = SkillQueue[item];
		else
			SkillQueueTail.next = SkillQueue[item];
		end
		SkillQueueTail = SkillQueue[item];
	end
	SkillQueue[item].amount = amount;

	UIErrorsFrame:AddMessage("Queueing " .. amount .. " x [" .. item .. "]", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);

	P.Iterate();
end

function P.Iterate()
	if (CastingSkill) then return nil; end

	local index = P.GetTradeItemIndex(SkillQueueHead.item);
	if (index >= 0) then
		DoTradeSkill(index, SkillQueueHead.amount);
	end
end

function P.HandleCreated(item)
	if (SkillQueueHead and item == SkillQueueHead.item) then
		SkillQueueHead.amount = SkillQueueHead.amount - 1;
		if (SkillQueueHead.amount <= 0) then
			CastingSkill = false; -- this will get set back to true when we get the SPELLCAST_START event
			SkillQueue[SkillQueueHead.item] = nil;
			SkillQueueHead = SkillQueueHead.next;
			if (SkillQueueHead) then
				P.Iterate();
			else
				SkillQueueTail = nil;
			end
		end
	end
end

function P.GetTradeSkillInfo(index)
	local skillName, difficulty, amount = GetTradeSkillInfo(index);
	local itemName = P.GetTradeItemName(index);

	return itemName, amount;
end

function P.GetTradeItemName(index)
	local linktext = GetTradeSkillItemLink(index);
	if (linktext) then
		local _,_,item = string.find(linktext, "^.*%[(.*)%].*$");
		return item;
	else
		local item, difficulty, amount = GetTradeSkillInfo(index);
		return item;
	end
end

-- find the trade skill index based on the item item.  this is necessary because sometimes the indices
-- change when your skill level increases.
function P.GetTradeItemIndex(itemName)
	local numSkills = GetNumTradeSkills();
	for index = 1,numSkills do
		local curName = P.GetTradeItemName(index);
		if (curName == itemName) then
			return index;
		end
	end
	return -1
end
